unit feeds;

interface

uses
  ComObj,activex,ShellApi, Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Inifiles, Dialogs, StdCtrls,utils, ExtCtrls, ComCtrls,
  splashscreen,strutils,IdHTTPHeaderInfo,samplefeeds,generic,keywordsets,
  shortintmap,compmap;

const
   MIN_REFRESH_VALUE = 60;
   defaultRefreshInterval = 300;

type

  TfrmFeeds = class(TForm)
    Panel2: TPanel;
    Panel4: TPanel;
    Panel5: TPanel;
    lblAddFeedURL: TLabel;
    txtAddFeedURL: TEdit;
    btnAddFeed: TButton;
    Splitter1: TSplitter;
    Panel3: TPanel;
    Panel7: TPanel;
    lblRefreshInterval: TLabel;
    cbActive: TCheckBox;
    txtRefreshInterval: TEdit;
    pnlKeywords: TPanel;
    Panel9: TPanel;
    Feeds: TLabel;
    lbFeeds: TListBox;
    btnAllInactive: TButton;
    btnAllActive: TButton;
    btnDelete: TButton;
    btnOpenFeedInBrowser: TButton;
    lblShowRelevance: TLabel;
    tbShowRelevance: TTrackBar;
    Label3: TLabel;
    cbCommonIntervals: TComboBox;
    btnHelp: TButton;
    btnSamples: TButton;
    lbFeedKeywordSets: TListBox;
    lbAvailableKeywordSets: TListBox;
    btnAddKeywordSetToFeed: TButton;
    btnRemoveKeywordSetFromFeed: TButton;
    Label2: TLabel;
    Label5: TLabel;
    btnManageKeywordSets: TButton;
    btnAddAllKeywordSetsToFeed: TButton;
    btnRemoveAllKeywordSetsFromFeed: TButton;
    btnSave: TButton;
    cbShowImages: TCheckBox;
    cbContinuousScan: TCheckBox;
    cbResetUnreadArticlesOnExit: TCheckBox;
    cbShowAllArticles: TCheckBox;
    btnScanNow: TButton;
    btnCheckRel: TButton;
    btnUnitTests: TButton;
    btnImportOPML: TButton;
    Label1: TLabel;
    lblLastScanTime: TLabel;
    btnScanAllNow: TButton;
    procedure btnAddFeedClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure lbFeedsClick(Sender: TObject);
    procedure cbActiveClick(Sender: TObject);
    procedure txtRefreshIntervalChange(Sender: TObject);
    procedure txtAddFeedURLChange(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnAllActiveClick(Sender: TObject);
    procedure btnAllInactiveClick(Sender: TObject);
    procedure btnOpenFeedInBrowserClick(Sender: TObject);
    procedure btnUnitTestsClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure btnCheckRelClick(Sender: TObject);
    procedure tbShowRelevanceChange(Sender: TObject);
    procedure txtRefreshIntervalExit(Sender: TObject);
    procedure txtAddFeedURLClick(Sender: TObject);
    procedure txtRefreshIntervalEnter(Sender: TObject);
    procedure cbCommonIntervalsChange(Sender: TObject);
    procedure cbCommonIntervalsExit(Sender: TObject);
    procedure btnSamplesClick(Sender: TObject);
    procedure btnManageKeywordSetsClick(Sender: TObject);
    procedure btnAddKeywordSetToFeedClick(Sender: TObject);
    procedure btnRemoveKeywordSetFromFeedClick(Sender: TObject);
    procedure btnAddAllKeywordSetsToFeedClick(Sender: TObject);
    procedure btnRemoveAllKeywordSetsFromFeedClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnSaveClick(Sender: TObject);
    procedure cbShowImagesClick(Sender: TObject);
    procedure cbContinuousScanClick(Sender: TObject);
    procedure cbResetUnreadArticlesOnExitClick(Sender: TObject);
    procedure cbShowAllArticlesClick(Sender: TObject);
    procedure btnScanNowClick(Sender: TObject);
    procedure btnHelpClick(Sender: TObject);
    procedure btnImportOPMLClick(Sender: TObject);
    procedure btnScanAllNowClick(Sender: TObject);
  private
    { Private declarations }
    dataChanged, shownHelp: boolean;
    procedure dataHasChanged;
    procedure sortFeeds;
    procedure updateFeedDetails;
    procedure setAllActive(active: boolean);
    function findFeedByUrl(findUrl: string): integer;
    procedure showKeywordSetsForFeed(var feed: TFeed);
  public
    { Public declarations }
    procedure checkSaveConfig;
  end;


var
  frmFeeds: TfrmFeeds;

implementation

uses main;


{$R *.dfm}



procedure TfrmFeeds.btnAddAllKeywordSetsToFeedClick(Sender: TObject);
begin
   lastUserInteraction := CurrentTimeMillis;

   if lbFeeds.itemIndex = -1 then
   begin
      ShowMessage('Please select a feed on the left');
      exit;
   end;

   if lbAvailableKeywordSets.Count = 0 then
   begin
      ShowMessage('No more available keyword sets exist.  Click Manage if you want to create some.');
      exit;
   end;

   while lbAvailableKeywordSets.items.Count > 0 do
   begin
      lbFeedKeywordSets.Items.Add(lbAvailableKeywordSets.Items[0]);
      lbAvailableKeywordSets.items.Delete(0);
   end;

   updateFeedDetails;
end;

procedure TfrmFeeds.btnAddFeedClick(Sender: TObject);
var
   i: integer;
begin

   // Firefox always sends URLS as:  feed://newsrss.bbc.co.uk/rss/newsonline_uk_edition/front_page/rss.xml
   txtAddFeedURL.Text := StringReplace(txtAddFeedURL.Text, 'feed://', 'http://', [rfReplaceAll, rfIgnoreCase]);

   if (uppercase(copy(txtAddFeedURL.Text, 1, 7)) <> 'HTTP://') and
      (uppercase(copy(txtAddFeedURL.Text, 1, 8)) <> 'HTTPS://') then
   begin
      ShowMessage('Feed URL must start with HTTP:// or HTTPS://');
      txtAddFeedURL.SetFocus;
      exit;
   end;

   // check feed doesn't already exist
   for i := 0 to lbFeeds.Items.Count-1 do
   begin
      if uppercase(lbFeeds.Items[i]) = uppercase(txtAddFeedURL.Text) then
      begin
         ShowMessage('You are already subscribed to this feed');
         exit;
      end;
   end;

   lbFeeds.items.add(txtAddFeedURL.Text);
   setLength(feedsList, length(feedsList)+1);
   with feedsList[length(feedsList)-1] do
   begin
      url := txtAddFeedURL.Text;
      active := true;
      showAllArticles := false;
      keywordSets := TStringList.create;
      try
         refreshInterval := StrToInt(txtRefreshInterval.text);
      except
         refreshInterval := defaultRefreshInterval;
      end;
      lastRefresh := 0;
   end;

   sortFeeds;

   // select the new feed
   for i := 0 to lbFeeds.Items.Count-1 do
   begin
      if lbFeeds.Items[i] = txtAddFeedURL.Text then
      begin
         lbFeeds.ItemIndex := i;
         break;
      end;
   end;
   lbFeedsClick(self);

   dataHasChanged;
end;

procedure TfrmFeeds.btnAddKeywordSetToFeedClick(Sender: TObject);
begin
   lastUserInteraction := CurrentTimeMillis;

   if lbFeeds.itemIndex = -1 then
   begin
      ShowMessage('Please select a feed on the left');
      exit;
   end;

   if lbAvailableKeywordSets.Count = 0 then
   begin
      ShowMessage('No more available keyword sets exist.  Click Manage if you want to create some.');
      exit;
   end;

   if lbAvailableKeywordSets.itemIndex = -1 then
   begin
      ShowMessage('Please select a keyword set from the list on the right.');
      exit;
   end;

   lbFeedKeywordSets.Items.Add(lbAvailableKeywordSets.Items[lbAvailableKeywordSets.itemIndex]);
   lbAvailableKeywordSets.items.Delete(lbAvailableKeywordSets.itemIndex);
   updateFeedDetails;
end;

procedure TfrmFeeds.sortFeeds;
var
   i,j: integer;
   tmpFeed: TFeed;
begin
   Screen.Cursor := crHourglass;
   for i := 0 to length(feedsList)-1 do
   begin
      for j := 0 to length(feedsList)-1 do
      begin
         if feedsList[i].url < feedsList[j].url then
         begin
            tmpFeed := feedsList[i];
            feedsList[i] := feedsList[j];
            lbFeeds.items[i] := feedsList[j].url;
            feedsList[j] := tmpFeed;
            lbFeeds.items[j] := tmpFeed.url;
         end;
      end;
   end;
   Screen.Cursor := crDefault;
end;

procedure TfrmFeeds.btnAllActiveClick(Sender: TObject);
begin
   setAllActive(true);
end;

procedure TfrmFeeds.btnAllInactiveClick(Sender: TObject);
begin
   setAllActive(false);
end;


procedure TfrmFeeds.setAllActive(active: boolean);
var
   i: integer;
begin
   for i := 0 to lbFeeds.Count-1 do
      feedsList[i].active := active;

   cbActive.Checked := active;
   dataHasChanged;
end;

procedure TfrmFeeds.btnDeleteClick(Sender: TObject);
var
   i: integer;
begin
   try
      if lbFeeds.ItemIndex > -1 then
      begin
         if MessageDlg('Delete feed?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
         begin
            for i := lbFeeds.ItemIndex to lbFeeds.Count-2 do
               feedsList[i] := feedsList[i+1];
            setLength(feedsList, length(feedsList)-1);
            lbFeeds.Items.Delete(lbFeeds.ItemIndex);
            dataHasChanged;
         end;
      end
      else
         ShowMessage('Please select a feed to delete');
   finally
   end;
end;

procedure TfrmFeeds.btnHelpClick(Sender: TObject);
begin
   ShellExecute(GetDesktopWindow, 'open', PChar(mainWuulHost + 'pheedstorm/readme.html'), nil, nil, SW_ShowNormal)
end;

procedure TfrmFeeds.btnImportOPMLClick(Sender: TObject);
var
   f: textfile;
   s,newFeedUrl: string;
   p1, p2, added: integer;
begin
   added := 0;
   with TOpenDialog.create(self) do
   begin
      try
         filter := 'All files (*.*)|*.*';
         if not execute then exit;
         AssignFile(f, filename);
         reset(f);
         try
            while not eof(f) do
            begin
               readln(f,s);
               p1 := pos('XMLURL="HTTP://', uppercase(s));
               if p1 > 0 then
               begin
                  p2 := posex('"', s, p1+8);
                  if p2 > 0 then
                  begin
                     newFeedUrl := copy(s,p1+8,p2-p1-8);
                     if findFeedByUrl(newFeedUrl) = -1 then
                     begin
                        lbFeeds.items.add(newFeedUrl);
                        setLength(feedsList, length(feedsList)+1);
                        with feedsList[length(feedsList)-1] do
                        begin
                           url := newFeedUrl;
                           active := true;
                           keywordSets := TStringList.create;
                           refreshInterval := defaultRefreshInterval;
                           lastRefresh := 0;
                        end;
                        inc(added);
                     end;
                  end;
               end;
            end;
         finally
            closefile(f);
         end;

         if added > 0 then
         begin
            SortFeeds;
            dataHasChanged;
         end;

         ShowMessage(IntToStr(added) + ' feed(s) added');
      finally
         free;
      end;
   end;
end;

procedure TfrmFeeds.btnOpenFeedInBrowserClick(Sender: TObject);
begin
   lastUserInteraction := CurrentTimeMillis;
   if lbFeeds.ItemIndex > -1 then
      ShellExecute(GetDesktopWindow, 'open', PChar(feedsList[lbFeeds.ItemIndex].url), nil, nil, SW_ShowNormal)
   else
      ShowMessage('Please select a feed to open');
end;

procedure TfrmFeeds.btnRemoveAllKeywordSetsFromFeedClick(Sender: TObject);
begin
   lastUserInteraction := CurrentTimeMillis;

   if lbFeeds.itemIndex = -1 then
   begin
      ShowMessage('Please select a feed on the left');
      exit;
   end;

   while lbFeedKeywordSets.items.Count > 0 do
   begin
      lbAvailableKeywordSets.Items.Add(lbFeedKeywordSets.Items[0]);
      lbFeedKeywordSets.items.Delete(0);
   end;
   
   updateFeedDetails;
end;

procedure TfrmFeeds.btnRemoveKeywordSetFromFeedClick(Sender: TObject);
begin
   lastUserInteraction := CurrentTimeMillis;

   if lbFeeds.itemIndex = -1 then
   begin
      ShowMessage('Please select a feed on the left');
      exit;
   end;

   if lbFeedKeywordSets.itemIndex = -1 then
   begin
      ShowMessage('Please select a keyword set to remove from the list on the left.');
      exit
   end;

   lbAvailableKeywordSets.Items.Add(lbFeedKeywordSets.Items[lbFeedKeywordSets.itemIndex]);
   lbFeedKeywordSets.items.Delete(lbFeedKeywordSets.itemIndex);
   updateFeedDetails;
end;

procedure TfrmFeeds.btnSamplesClick(Sender: TObject);
var
   i, j: integer;
begin
   with TfrmSampleFeeds.create(self) do
   begin
      try
         setFeeds(feedsList);
         ShowModal;
         if ModalResult = mrOk then
         begin
            // add any new items
            for i := 0 to lbYourFeeds.items.count-1 do
            begin
               if findFeedByUrl(lbYourFeeds.Items[i]) = -1 then
               begin
                  lbFeeds.items.add(lbYourFeeds.Items[i]);
                  setLength(feedsList, length(feedsList)+1);
                  with feedsList[length(feedsList)-1] do
                  begin
                     url := lbYourFeeds.Items[i];
                     active := true;
                     keywordSets := TStringList.create;
                     refreshInterval := defaultRefreshInterval;
                     lastRefresh := 0;
                  end;
               end;
            end;

            // remove any feeds no longer in the list
            for i := length(feedsList)-1 downto 0 do
            begin
               if lbYourFeeds.items.IndexOf(feedsList[i].url) = -1 then
               begin
                  for j := i to lbFeeds.Count-2 do
                     feedsList[j] := feedsList[j+1];
                  setLength(feedsList, length(feedsList)-1);
                  lbFeeds.Items.Delete(i);
               end;
            end;

            sortFeeds;
            dataHasChanged;
         end;
      finally
         Free;
      end;
   end;
end;

procedure TfrmFeeds.btnSaveClick(Sender: TObject);
begin
   frmMain.saveAllConfig;
   dataChanged := false;
   btnSave.Enabled := false;
end;

procedure TfrmFeeds.btnScanAllNowClick(Sender: TObject);
var
   i: integer;
begin
   for i := 0 to lbFeeds.Count-1 do
      feedsList[i].lastRefresh := 0;

   interrupted := true;
end;

procedure TfrmFeeds.btnScanNowClick(Sender: TObject);
begin
   if lbFeeds.itemIndex > -1 then
   begin
      feedsList[lbFeeds.itemIndex].lastRefresh := 0;
      interrupted := true;
   end;
end;

function TfrmFeeds.findFeedByUrl(findUrl: string): integer;
var
   i: integer;
begin
   for i := 0 to length(feedsList)-1 do
   begin
      if feedsList[i].url = findUrl then
      begin
         result := i;
         exit;
      end;
   end;
   result := -1;
end;



procedure TfrmFeeds.btnUnitTestsClick(Sender: TObject);
var
   i: integer;
   wordList,cleanWords, sorted: TStringList;
   // feed: TFeed;
   sim: TShortIntMap;
begin
   verboseOutput := true;

   // dumpLinkTable;
   // exit;

   try
      sim := TShortIntMap.create;
      sim.put('EEE',5);
      sim.put('AAA',1);
      sim.put('CCC',3);
      sim.put('BBB',2);
      sim.put('DDD',4);
      assert(sim.get('EEE') = 5);
      assert(sim.get('AAA') = 1);
      assert(sim.get('CCC') = 3);
      assert(sim.get('DDD') = 4);
      assert(sim.get('BBB') = 2);
      sim.free;

      assert(removeExcessSpaces('  test test    test     test                 test  ') = 'test test test test test');
      assert(removeExcessSpaces('test test') = 'test test');
      assert(removeExcessSpaces('test') = 'test');

      cleanWords := mainThread.getCleanWordList('   and  here''s to john''s apples for the monsters'' party  ');
      // showmessage(cleanWords.getText);
      assert(cleanWords.getText = 'JOHN'+chr(13)+chr(10)+'APPLE'+chr(13)+chr(10)+'MONSTER'+chr(13)+chr(10)+'PARTY'+chr(13)+chr(10));
      cleanWords.free;

      cleanWords := mainThread.getCleanWordList('in john''s $$$ "report" he said that if "iran''s nuclear programme doesn''t stop soon george bush will get ''really'' cross"!!!, so thEn everyone everyone everyone reads it john''s report report!');
      removeDuplicates(cleanWords);
      // showmessage(cleanWords.getText);
      assert(cleanWords.getText = 'BUSH'+CHR(13)+CHR(10)+'CROSS'+CHR(13)+CHR(10)+'EVERYONE'+CHR(13)+CHR(10)+'GEORGE'+CHR(13)+CHR(10)+'GET'+CHR(13)+CHR(10)+'IRAN'+CHR(13)+CHR(10)+'JOHN'+CHR(13)+CHR(10)+'NUCLEAR'+CHR(13)+CHR(10)+'PROGRAMME'+CHR(13)+CHR(10)+'READ'+CHR(13)+CHR(10)+'REALLY'+CHR(13)+CHR(10)+'REPORT'+CHR(13)+CHR(10)+'SOON'+CHR(13)+CHR(10)+'STOP'+CHR(13)+CHR(10));
      cleanWords.free;

      sorted := TStringList.create;
      sorted.Sorted := true;
      sorted.CaseSensitive := false;
      sorted.add('blah');
      sorted.add('test');
      sorted.add('aaa');
      assert(findStringInSortedList('test',sorted));
      assert(findStringInSortedList('AAA',sorted));
      assert(findStringInSortedList('blah',sorted));
      assert(not findStringInSortedList('xblah',sorted));
      sorted.free;

      assert(removeHtml('bar><nobar>',false) = '');
      assert(removeHtml('<nobr',false) = '');
      assert(removeHtml('nobr>',false) = '');
      assert(removeHtml('<<nobr',false) = '');
      assert(removeHtml('nobr><blah',false) = '');
      assert(removeHtml('test<br><p><script language=javascript>function x():</script>end&quot;',false) = 'test  end"');

      // scan for keyword tests
      {feed.keywords := parseString('gary mckinnon'+chr(13)+chr(10)+'richard,dawkins',chr(13)+chr(10));;
      assert(scanKeywords(feed,'gary is a hacker') = '');
      assert(scanKeywords(feed,'gary mckinnon is a hacker') <> '');
      assert(scanKeywords(feed,'richard dawkins rocks') <> '');
      feed.keywords.free;}

      relevantWords.clear;
      setLength(wordLinks, 0);

      // positive test (max weight)
      for i := 1 to 100 do
         mainThread.updateRelevance(10,'jordan abdullah katie sand');
      frmMain.dumpLinkTable;
      wordList := mainThread.getCleanWordList('jordan katie jordan abdullah katie sand');
      assert(mainThread.checkRelevanceInRelTable(wordList) = 100);
      wordList.free;

      // zero test
      assert(mainThread.checkRelevance('now is the winter of our discontent') = 0);
      wordList := mainThread.getCleanWordList('now is the winter of our discontent');
      assert(mainThread.checkRelevanceInRelTable(wordList) = 0);
      wordList.free;

      // negative test (max weight)
      for i := 1 to 100 do
         mainThread.updateRelevance(-10,'jordan katie');
      frmMain.dumpLinkTable;
      wordList := mainThread.getCleanWordList('jordan katie katie');
      assert(mainThread.checkRelevanceInRelTable(wordList) = -100);
      wordList.free;

      // single word test
      wordList := mainThread.getCleanWordList('jordan');
      assert(mainThread.checkRelevanceInRelTable(wordList) = 0);
      wordList.free;

      // exclude common words
      wordList := mainThread.getCleanWordList('jordan AND THE jordan');
      assert(mainThread.checkRelevanceInRelTable(wordList) = 0);
      wordList.free;

      relevantWords.clear;
      setLength(wordLinks, 0);

      // deep word scan tests
      for i := 1 to 2 do
      begin
         mainThread.updateRelevance(9,'jordan abdullah');
         mainThread.updateRelevance(8,'jordan sand');
         mainThread.updateRelevance(-8,'jordan''s katie');
         mainThread.updateRelevance(-5,'katie sands');
         mainThread.updateRelevance(-3,'katie abdullah');
         mainThread.updateRelevance(7,'sand abdullah''s');
      end;

      frmMain.dumpLinkTable;

      assert(mainThread.checkRelevance('jordan abdullah') = 40);
      assert(mainThread.checkRelevance('abdullahs jordan') = 40);
      assert(mainThread.checkRelevance('jordan sands') = (((36+40) div 2) + 32) div 2);
      assert(mainThread.checkRelevance('sand jordan') = (((36+40) div 2) + 32) div 2);
      assert(mainThread.checkRelevance('jordan katie') = -14);
      assert(mainThread.checkRelevance('katies jordan') = -14);
      assert(mainThread.checkRelevance('jordan katie sand abdullah') = 10);
      assert(mainThread.checkRelevance('abdullah katie''s sand jordan') = 10);

      // FIXME save new links table & load it back
      
      // test save
      {Save2DShortArray(relTable, 'c:\temp\reltable.dat');
      relevantWords.saveToFile('c:\temp\rel.dat');
      relevantWords.clear;
      setLength(relTable, 0, 0);}

      // test load
      {relevantWords.loadFromFile('c:\temp\rel.dat');
      Load2DShortArray(relTable, 'c:\temp\reltable.dat');
      DeleteFile('c:\temp\rel.dat');
      DeleteFile('c:\temp\reltable.dat');}

      // retest data
      frmMain.cleanData;
      
      assert(mainThread.checkRelevance('jordan abdullah') = 40);
      assert(mainThread.checkRelevance('abdullah jordan') = 40);
      assert(mainThread.checkRelevance('jordans sand') = (((36+40) div 2) + 32) div 2);
      assert(mainThread.checkRelevance('sand jordan') = (((36+40) div 2) + 32) div 2);
      assert(mainThread.checkRelevance('jordan katie') = -14);
      assert(mainThread.checkRelevance('katie''s jordan') = -14);
      assert(mainThread.checkRelevance('jordan katie sand abdullah') = 10);
      assert(mainThread.checkRelevance('abdullah katie sand jordan') = 10);

      // long sentence tests
      relevantWords.clear;
      setLength(wordLinks, 0);

      for i := 1 to 5 do
         mainThread.updateRelevance(10,'the quick brown fox jumped over the big fat bloated lazy dog ');

      wordList := mainThread.getCleanWordList('quick brown fox jumped over big fat bloated lazy dog');
      assert(mainThread.checkRelevanceInRelTable(wordList) = 100);
      wordList.free;

      {wordList := getCleanWordList('quick brown wolf jumped over big fat bloated lazy cat');
      assert(checkRelevanceInRelTable(wordList) = 62);
      wordList.free;}

      showmessage('All tests passed');

   finally
      application.terminate; // otherwise we will overwrite the data when we exit
   end;

end;


procedure TfrmFeeds.btnManageKeywordSetsClick(Sender: TObject);
var
   frmKeywordSets: TfrmKeywordSets;
   i,j,k: integer;
   found: boolean;
   feed: TFeed;
begin
   lastUserInteraction := CurrentTimeMillis;
   frmKeywordSets := TfrmKeywordSets.create(self);
   try
      // show all keyword sets on form
      setLength(frmKeywordSets.keywordSetList,length(keywordSetList));
      for i := 0 to length(frmKeywordSets.keywordSetList)-1 do
      begin
         with frmKeywordSets.keywordSetList[i] do
         begin
            name := keywordSetList[i].name;
            keywords := TStringList.create;
            keywords.AddStrings(keywordSetList[i].keywords);
         end;
         frmKeywordSets.lbKeywordSets.Items.add(keywordSetList[i].name);
      end;

      if frmKeywordSets.lbKeywordSets.Items.count > 0 then
      begin
         frmKeywordSets.lbKeywordSets.ItemIndex := 0;
         frmKeywordSets.lbKeywordSets.OnClick(nil);
         frmKeywordSets.txtKeywords.Enabled := true;
      end
      else
      begin
         frmKeywordSets.txtKeywords.Enabled := false;
      end;

      if firstTime then ShowMessage('I have set up some example keyword sets, please change these to suit your own criteria');
      frmKeywordSets.ShowModal;
      if frmKeywordSets.modalResult = mrOk then
      begin
         // update keyword sets using the new form data
         setLength(keywordSetList,length(frmKeywordSets.keywordSetList));
         for i := 0 to length(keywordSetList)-1 do
         begin
            with keywordSetList[i] do
            begin
               name := frmKeywordSets.keywordSetList[i].name;
               keywords := TStringList.create;
               keywords.AddStrings(frmKeywordSets.keywordSetList[i].keywords);
            end;
         end;

         // for all feeds, remove any keywords sets that were deleted
         for i := 0 to length(feedsList)-1 do
         begin
            for j := feedsList[i].keywordSets.Count-1 downto 0 do
            begin
               found := false;
               for k := 0 to length(keywordSetList)-1 do
               begin
                  if feedsList[i].keywordSets[j] = keywordSetList[k].name then
                  begin
                     found := true;
                     break;
                  end;
               end;
               if not found then
               begin
                  // ShowMessage('Removing keyword set ' + feedsList[i].keywordSets[j] + ' from feed ' + feedsList[i].url);
                  feedsList[i].keywordSets.Delete(j);
               end;
            end;
         end;

         // update this feed's keyword sets
         if lbFeeds.itemIndex > -1 then
         begin
            feed := feedsList[lbFeeds.itemIndex];
            showKeywordSetsForFeed(feed);
         end;

         dataHasChanged;
      end;
   finally
      frmKeywordSets.free;
      lastUserInteraction := CurrentTimeMillis;
   end;
end;

procedure TfrmFeeds.btnCheckRelClick(Sender: TObject);
var
   words: string;
   wordsList: TStringList;
begin
   words := InputBox(self.caption, 'Enter word(s) to check separated by spaces','');
   wordsList := mainThread.getCleanWordList(words);
   try
      showmessage('Simple relevance='+inttostr(mainThread.checkSimpleRelevance(wordsList))+'%, ' +
         'Relationship relevance='+inttostr(mainThread.checkRelevanceInRelTable(wordsList))+'%, ' +
         'Overall relevance='+inttostr(mainThread.checkRelevance(words))+'%');
   finally
      wordsList.free;
   end;
end;



procedure TfrmFeeds.tbShowRelevanceChange(Sender: TObject);
begin
   dataHasChanged;
end;



procedure TfrmFeeds.cbActiveClick(Sender: TObject);
begin
   if lbFeeds.ItemIndex > -1 then updateFeedDetails;
end;

procedure TfrmFeeds.cbCommonIntervalsChange(Sender: TObject);
var
   p1, p2: integer;
begin
   p1 := pos('(', cbCommonIntervals.Text);
   p2 := pos(')', cbCommonIntervals.Text);
   txtRefreshInterval.Text := copy(cbCommonIntervals.Text, p1+1, p2-p1-1);
   dataHasChanged;
   updateFeedDetails;
end;

procedure TfrmFeeds.cbCommonIntervalsExit(Sender: TObject);
begin
   cbCommonIntervals.Visible := false;
end;


procedure TfrmFeeds.cbContinuousScanClick(Sender: TObject);
begin
   frmMain.continuousScan:= cbContinuousScan.checked;
   dataHasChanged;
end;

procedure TfrmFeeds.cbResetUnreadArticlesOnExitClick(Sender: TObject);
begin
   dataHasChanged;
   frmMain.resetUnreadArticlesOnExit := cbResetUnreadArticlesOnExit.Checked;
end;

procedure TfrmFeeds.dataHasChanged;
begin
   if not initialised then exit;
   lastUserInteraction := CurrentTimeMillis;
   dataChanged := true;
   btnSave.Enabled := true;
   interrupted := true;
end;

procedure TfrmFeeds.cbShowAllArticlesClick(Sender: TObject);
begin
   if lbFeeds.ItemIndex > -1 then updateFeedDetails;
end;

procedure TfrmFeeds.cbShowImagesClick(Sender: TObject);
begin
   frmMain.showImages:= cbShowImages.checked;
   frmMain.setLinkPanelHeight;
   dataHasChanged;
end;

procedure TfrmFeeds.txtAddFeedURLChange(Sender: TObject);
begin
   lastUserInteraction := CurrentTimeMillis;
   if not updatingGUI then
   begin
      lbFeeds.itemIndex := -1; // deselect so we don't overwrite the selected item
      cbActive.Checked := true;
      txtRefreshInterval.Text := IntToStr(defaultRefreshInterval);
   end;
end;

procedure TfrmFeeds.txtAddFeedURLClick(Sender: TObject);
begin
   lastUserInteraction := CurrentTimeMillis;
end;

procedure TfrmFeeds.txtRefreshIntervalChange(Sender: TObject);
begin
   lastUserInteraction := CurrentTimeMillis;
end;


procedure TfrmFeeds.txtRefreshIntervalEnter(Sender: TObject);
var
   i: integer;
begin
   cbCommonIntervals.Visible := true;
   for i := 0 to cbCommonIntervals.Items.Count-1 do
   begin
      if pos('('+ txtRefreshInterval.text+')', cbCommonIntervals.Items[i]) > 0 then
      begin
         cbCommonIntervals.ItemIndex := i;
         exit;
      end;
   end;
end;

procedure TfrmFeeds.txtRefreshIntervalExit(Sender: TObject);
begin
   updateFeedDetails;
end;

procedure TfrmFeeds.updateFeedDetails;
var
   i: integer;
begin
   if (not updatingGUI) and (lbFeeds.itemIndex > -1) then
   begin
      dataHasChanged;
      with feedsList[lbFeeds.itemIndex] do
      begin
         active := cbActive.Checked;
         btnScanNow.Enabled := active;
         showAllArticles := cbShowAllArticles.Checked;
         keywordSets.Clear;
         for i := 0 to lbFeedKeywordSets.Count-1 do
            keywordSets.add(lbFeedKeywordSets.Items[i]);
         try
            refreshInterval := StrToInt(txtRefreshInterval.text);
            if refreshInterval < MIN_REFRESH_VALUE then raise Exception.create('Invalid refresh value');
         except
            refreshInterval := MIN_REFRESH_VALUE;
         end;
      end;
      interrupted := true;
   end;
end;


procedure TfrmFeeds.FormActivate(Sender: TObject);
begin
   if (FirstTime) and (not shownHelp) then
   begin
      shownHelp := true;
      ShowMessage('You can add/remove feeds using the controls at the top of this page.' + chr(13)+chr(10) + chr(13)+chr(10) +
      'When you have set up your feeds, use the controls on the right of the page to configure keyword monitoring.'+ chr(13)+chr(10) + chr(13)+chr(10) +
      'Once articles start appearing, try to rate as many as you can which is essential for the learning process to work effectively.'+ chr(13)+chr(10) +
      'You can then adjust the slider at the top-right to control the number of articles that are displayed.');

      if MessageDlg('Would you like to select from a list of sample RSS feeds?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
         btnSamplesClick(self);

   end;

end;


procedure TfrmFeeds.checkSaveConfig;
begin
   if dataChanged then
   begin
      if MessageDlg('Save config changes?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
      begin
         btnSaveClick(self);
      end;
   end;
end;

procedure TfrmFeeds.FormClose(Sender: TObject; var Action: TCloseAction);
begin
   checkSaveConfig;
end;

procedure TfrmFeeds.FormCreate(Sender: TObject);
begin
   dataChanged := false;
   btnSave.Enabled := false;
   cbCommonIntervals.DropDownCount := cbCommonIntervals.Items.Count;
   lblShowRelevance.Hint := 'As you provide feedback on each article''s relevance, Pheedstorm will learn about the kind of articles you find interesting.' + chr(13) + chr(10) +
      'You can use this slider to indicate how relevant an article needs to be before Pheedstorm decides to show it to you.' + chr(13) + chr(10) +
      'If you move the slider towards LOW, you will get lots of articles that may have little relevance.' + chr(13) + chr(10) +
      'However, if you move the slider towards HIGH, Pheedstorm will only show you articles it thinks are very relevant.' + chr(13) + chr(10) +
      'Do not set it to either extreme though or you will get every single article or none at all.' + chr(13) + chr(10) +
      'Try starting off by setting it fairly low and adjusting it upwards gradually.';
   tbShowRelevance.Hint := lblShowRelevance.Hint;
   label3.Hint := lblShowRelevance.Hint;
   txtRefreshInterval.Text := IntToStr(MIN_REFRESH_VALUE);
   lblRefreshInterval.Caption := lblRefreshInterval.Caption + IntToStr(MIN_REFRESH_VALUE) + ')';
end;


procedure TfrmFeeds.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
   // note - form's "KeyPreview" must be set to true or this event isn't fired
   btnUnitTests.visible := ((ssShift in Shift) and (ssAlt in Shift));
   btnCheckRel.visible := ((ssShift in Shift) and (ssAlt in Shift));
end;

procedure TfrmFeeds.lbFeedsClick(Sender: TObject);
var
   feed: TFeed;
begin
   lastUserInteraction := CurrentTimeMillis;
   cbCommonIntervals.Visible := false;
   updatingGUI := true; // so we don't fire the OnChange events
   try
      feed := feedsList[lbFeeds.itemIndex];
      txtAddFeedURL.Text := feed.url;
      txtRefreshInterval.text := IntToStr(feed.refreshInterval);
      showKeywordSetsForFeed(feed);
      cbActive.Checked := feed.active;
      cbShowAllArticles.Checked := feed.showAllArticles;
      if feed.lastRefresh > 0 then
         lblLastScanTime.Caption := DateTimeToStr(javaTimeToTDateTime(feed.lastRefresh))
      else
         lblLastScanTime.Caption := 'Never';
   finally
      updatingGUI := false;
   end;
end;

procedure TfrmFeeds.showKeywordSetsForFeed(var feed: TFeed);
var
   i: integer;
begin
   lbAvailableKeywordSets.Clear;
   lbFeedKeywordSets.Clear;
   for i := 0 to length(keywordSetList)-1 do
   begin
      if feed.keywordSets.IndexOf(keywordSetList[i].name) > -1 then
         lbFeedKeywordSets.items.add(keywordSetList[i].name)
      else
         lbAvailableKeywordSets.items.add(keywordSetList[i].name);
   end;
end;


end.

