unit newsticker;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, TlHelp32, ExtCtrls, inifiles, ComCtrls,math, utils;

  
// use GlobalMemoryStatusEx instead of GlobalMemoryStatus which is limited to 2GB RAM
type
  TMemoryStatusEx = packed record
    dwLength: DWORD;
    dwMemoryLoad: DWORD;
    ullTotalPhys: Int64;
    ullAvailPhys: Int64;
    ullTotalPageFile: Int64;
    ullAvailPageFile: Int64;
    ullTotalVirtual: Int64;
    ullAvailVirtual: Int64;
    ullAvailExtendedVirtual: Int64;
  end;

  function GlobalMemoryStatusEx(var lpBuffer: TMemoryStatusEx): BOOL; stdcall; external kernel32; // Function call for GlobalMemoryStatusEx


type

   TTickerThread = class(TThread)
  private
   frmOwner: TForm;
   isRunning: boolean;
   articlePos: integer;
  protected
    procedure Execute; override;
  public
    constructor create(_frmOwner: TForm);
    destructor Destroy; override;
    function isThreadRunning: boolean;
  end;

  TfrmNewsTicker = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
    tickerThread: TTickerThread;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure showMemoryStatus;
  public
    { Public declarations }
    text1, text2: string;
    procedure updateTicker;
  end;

const
   gap = 5;

var
  frmNewsTicker: TfrmNewsTicker;
  displayingText: boolean;
  pos1, pos2: integer;
  textSize: integer;
  maxMemoryOnThisPC: integer; // in MB

implementation

{$R *.dfm}


uses main;

procedure TfrmNewsTicker.CreateParams(var Params: TCreateParams);
begin
   // this ensures the form stays on top
   inherited CreateParams(Params);
   with Params do begin
      ExStyle := ExStyle or WS_EX_TOPMOST;
      WndParent := GetDesktopwindow;
   end;
end;


procedure TfrmNewsTicker.FormClose(Sender: TObject; var Action: TCloseAction);
begin
   frmMain.freeNewsTicker;
end;

procedure TfrmNewsTicker.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
   displayingText := false;
   tickerThread.Terminate;
   while tickerThread.isRunning do
   begin
      application.processmessages;
      sleep(500);
   end;
   CanClose := true;
end;

procedure TfrmNewsTicker.FormCreate(Sender: TObject);
var
   memoryStatus: TMemoryStatusEx;
begin
   label1.font.size := fontSize;
   label2.font.size := fontSize;

   self.ClientHeight := round(label1.height * 1.3);

   label1.left := self.width;
   label2.left := self.width * 3;

   pos1 := label1.left;
   pos2 := label2.left;

   FormResize(self);

   memoryStatus.dwLength := SizeOf(memoryStatus);
   GlobalMemoryStatusEx(memoryStatus);
   maxMemoryOnThisPC := Round(memoryStatus.ullTotalPhys / 1024 / 1024);

   tickerThread := TTickerThread.create(self);
   tickerThread.Priority := tpLower;
   tickerThread.FreeOnTerminate := true;
   tickerThread.resume;
end;

procedure TfrmNewsTicker.FormResize(Sender: TObject);
var
   tmp: string;
begin
   // labels should always appear in (vertical) centre of form
   label1.Top := (self.ClientHeight div 2) - (label1.height div 2);
   label2.Top := label1.Top;

   // calculate number of characters that will take up the display
   // NB: font must be fixed-width e.g. Courier
   tmp := '';
   repeat
      tmp := tmp + 'O';
   until label1.Canvas.TextWidth(tmp) >= self.Width;
   textSize := length(tmp);
end;

procedure TfrmNewsTicker.updateTicker;
begin
   label1.caption := text1;
   label2.caption := text2;

   // move both labels left by 1 pixel
   label1.Left := label1.Left-1;
   label2.Left := label2.Left-1;

   // as we hit the left-hand side, position the other label
   // either 1 pixel past the right side of the screen
   // or at the end of this label (whichever is greater)
   // as soon as the other label hits the right side
   // it will re-analyse the list of articles
   if label1.Left = 0 then
   begin
      label2.left := max(self.width+1, label1.left + label1.Canvas.TextWidth(text1));
   end;
   if label2.Left = 0 then
   begin
      label1.left := max(self.width+1, label2.left + label2.Canvas.TextWidth(text2));
   end;

   // record the positions so that the thread can access them
   // without directly referencing the GUI components
   pos1 := label1.left;
   pos2 := label2.left;

   if pos1 mod 5 = 0 then showMemoryStatus;
end;



procedure TfrmNewsTicker.showMemoryStatus;
var
   memoryStatus: TMemoryStatusEx;
   usedMemoryAmount, usedMemoryPercentage: integer; // in MB
begin
   memoryStatus.dwLength := SizeOf(memoryStatus);
   GlobalMemoryStatusEx(memoryStatus);
   usedMemoryAmount := maxMemoryOnThisPC - Round((memoryStatus.ullAvailPhys / 1024 / 1024));
   usedMemoryPercentage := (usedMemoryAmount * 100) div maxMemoryOnThisPC;
   self.Caption := 'News Ticker, memory ' + IntToStr(usedMemoryPercentage) + '% used';
   if usedMemoryPercentage > 95 then
      self.color := clRed
   else
      self.color := clBtnFace;
end;


{constructor}
constructor TTickerThread.create(_frmOwner: TForm);
begin
   inherited create(true); // create but don't start running yet
   frmOwner := _frmOwner;
   isRunning := true;
end;

{destructor}
destructor TTickerThread.Destroy;
begin
   inherited destroy;
end;


{main thread entry point}
procedure TTickerThread.Execute;
const
   updateInterval = 25;
var
   frmOwnerCast: TfrmNewsTicker;
   i: integer;
   tmpString: string;
   startLoopTime: comp;
   sleepTime: extended;
   showOnlyNewest: boolean;
   oldArticleCutoffTime: TDateTime;
begin
   try
      frmOwnerCast := (frmOwner as TfrmNewsTicker);
      articlePos := 0;
      while not terminated do
      begin
         startLoopTime := CurrentTimeMillis;

         // as soon as a label emerges from the right-hand side,
         // re-collect the list of unread articles
         if (pos1 = frmOwnerCast.width) or (pos2 = frmOwnerCast.width) then
         begin
            tmpString := '';

            // every other pass will only show the newest articles
            if showOnlyNewest then
               showOnlyNewest := false
            else
               showOnlyNewest := true;

            if showOnlyNewest then oldArticleCutoffTime := frmMain.getOldArticleCutoffTime;

            for i := 0 to length(articles) -1 do
            begin
               if (not articles[articlePos].read) and ((not showOnlyNewest) or (articles[articlePos].timestamp >= oldArticleCutoffTime)) then
                  tmpString := tmpString + removeHTML(articles[articlePos].title,true) + LeftPad(' ',' ',gap);
               inc(articlePos);
               if articlePos >= length(articles) then articlePos := 0;
               if length(tmpString) > textSize + 100 then break;
            end;

            if tmpString = '' then
            begin
               if showOnlyNewest then
                  tmpString := 'No new unread articles   '
               else
                  tmpString := 'No unread articles   ';
            end;

            // update the caption that is currently out-of-sight
            // but about to emerge from the right-hand side
            if pos1 = frmOwnerCast.width then
               frmOwnerCast.text1 := tmpString
            else if pos2 = frmOwnerCast.width then
               frmOwnerCast.text2 := tmpString;

         end;

         // try to keep a consistent update interval
         // i.e. subtracting time taken in article collection
         sleepTime := updateInterval - (CurrentTimeMillis - startLoopTime);
         if sleepTime > 0 then sleep(round(sleepTime));
         synchronize(frmOwnerCast.updateTicker);

      end;
   finally
      isRunning := false;
   end;
end;

function TTickerThread.isThreadRunning: boolean;
begin
   result := isRunning;
end;

end.
